package com.softslate.commerce.businessobjects.core;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.softslate.commerce.businessobjects.order.OrderItem;

/**
 * Interface for a utility object containing various methods used by
 * <code>BusinessProcessor</code>s in performing business logic.
 * <p>
 * Each <code>BusinessProcessor</code>s is able to retrieve an instance of
 * <code>BusinessObjectUtils</code> by calling its <code>utils</code> method.
 * <p>
 * The default "businessObjectUtilsImplementer" is
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectUtilsImpl}.
 * 
 * @author David Tobey
 * 
 */
public interface BusinessObjectUtils {

	/**
	 * Formats a Date object into a string suitable for storage in the database.
	 * 
	 * @param dateTime
	 * @return a String in the form of "yyyy-MM-dd HH:mm:ss:SSS" suitable for
	 *         storage in the database.
	 */
	abstract String formatDateTime(Date dateTime);

	/**
	 * Parses a string (typically taken from a database query) into a Date
	 * object.
	 * 
	 * @param dateTime
	 * @return a String in the form of "yyyy-MM-dd HH:mm:ss:SSS" suitable for
	 *         storage in the database.
	 */
	abstract Date parseDateTime(String dateTime);

	/**
	 * Formats a Double into a string suitable for displaying the double as a
	 * price, or a price representing 0 if the Double is null.
	 * 
	 * @param rawPrice
	 * @param currencyCode
	 * @param locale
	 * @return a String suitable for displaying the double as a price.
	 */
	abstract String formatPrice(Double rawPrice, String currencyCode,
			Locale locale);

	/**
	 * Formats a double into a string suitable for displaying the double as a
	 * price.
	 * 
	 * @param rawPrice
	 * @param currencyCode
	 * @param locale
	 * @return a String suitable for displaying the double as a price.
	 */
	abstract String formatPrice(double rawPrice, String currencyCode,
			Locale locale);

	/**
	 * Prepares a string, which may optionally be url-encoded, for use with an
	 * HTTP request, given a map of the request's parameters.
	 * 
	 * @param params
	 *            A Map of key-value String pairs representing the parameters of
	 *            the request.
	 * @param urlEncode
	 *            If true, the keys and values in the string will be
	 *            url-encoded.
	 * @return A String representing a valid URL-encoded query string.
	 * @throws Exception
	 */
	abstract String prepareRequestData(Map params, boolean urlEncode)
			throws Exception;

	/**
	 * Parses a query string of name value pairs into a Map.
	 * 
	 * @param response
	 *            A String key-value pairs in the form of a URL query string
	 * @return A Map representing the query string.
	 * @throws Exception
	 */
	abstract Map parseResponseData(String response) throws Exception;

	/**
	 * Sends an HTTP request using the POST method to a given URL.
	 * 
	 * @param urlString
	 *            The URL to send the request to.
	 * @param postData
	 *            A String representing a valid URL-encoded query string.
	 * @param followRedirect
	 *            A boolean indicating whether redirects should be followed.
	 * @param requestProperties
	 *            Map that holds request properties to be sent in connection.
	 * @param inputCharSet
	 *            Character set of the response data
	 * @return A String representing the server's response, or the Location
	 *         header if redirects are not to be followed.
	 * @throws Exception
	 */
	abstract String sendHTTPPost(String urlString, String postData,
			boolean followRedirect, Map requestProperties,
			String inputCharSet) throws Exception;

	/**
	 * Given a collection of bean objects returns a collection of
	 * <code>Map</code>s describing each bean's properties.
	 * 
	 * @param beans
	 * @return A <code>Collection</code> of <code>Map</code>s describing the
	 *         beans.
	 */
	abstract Collection getMapsFromBeans(Collection beans);

	/**
	 * Returns a DOM Level2 document that represents the given document.
	 * 
	 * @param xmlStream
	 *            An InputStream that is the source of the XML data
	 * @param ignoreWhitespace
	 *            This is passed through to the DocumentBuilder.
	 * 
	 */
	abstract Document parse(InputStream xmlStream, boolean ignoreWhitespace)
			throws ParserConfigurationException, SAXException, IOException;

	/**
	 * Parses a section of XML and returns the first Element node found
	 * 
	 * @param xml
	 *            The XML to be parsed in String form
	 * 
	 */
	public Node parsePartialDocument(String xml)
			throws ParserConfigurationException, SAXException, IOException;

	/**
	 * Determines if at least one element in a string array is greater than a
	 * given value.
	 * 
	 * @param paramValues
	 *            The string array being tested.
	 * @param value
	 *            The value for the string array's elements to be tested
	 *            against.
	 * @return <code>true</code> if at least one element in a string array is
	 *         greater than a given value. <code>false</code> if not.
	 */
	abstract boolean stringArrayValuesGreaterThan(String[] paramValues,
			int value);

	/**
	 * Rounds money values in general
	 * 
	 * @param value
	 *            The value to be rounded.
	 * @return The value after rounding
	 */
	abstract Double roundForMoney(Double value);

	/**
	 * Rounds money values in general
	 * 
	 * @param value
	 *            The value to be rounded.
	 * @return The value after rounding
	 */
	abstract Double roundForMoney(double value);

	/**
	 * From a given OrderItem, returns a Map of each of the OrderItemAttributes
	 * under it, keyed by the attributeCode.
	 * 
	 * @param oi
	 * @return A Map of OrderItemAttributes and their values.
	 */
	abstract Map buildOrderItemAttributeMap(OrderItem oi);

	/**
	 * Tests to see if a given order item has an OrderItemAttribute under it with the given attributeCode.
	 * 
	 * @param oi The order item
	 * @param attributeCode The attribute code
	 * @return true, if the OrderItem has an OrderItemAttribute under it with the attributeCode.
	 */
	abstract boolean doesOrderItemHaveAttributeCode(OrderItem oi,
			String attributeCode);

}
