package com.softslate.commerce.businessobjects.core;

import java.io.Serializable;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.events.ChangingSettingEvent;
import com.softslate.commerce.daos.core.SettingsDAO;

/**
 * Represents various settings used by the application. Settings for the
 * application are stored in the <code>sscSetting</code> database table. The
 * various settings represent everything from the default colors and fonts used
 * by the display templates, to the names of the Java classes that implement
 * various business object interfaces for the application.
 * <p>
 * An instance of <code>SettingsBean</code> is created in the Struts layer. It
 * is initialized there through {@link #initialize}, which populates the
 * {@link #map} property with the results of a query on the
 * <code>sscSetting</code> table. For each setting, an entry in the
 * <code>map</code> is created with the setting code used as the key and its
 * value as the key's value.
 * <p>
 * The <code>SettingsBean</code> instance created in the Struts layer is
 * passed throughout the application to look up needed settings. The
 * application's <code>BusinessObjectFactory</code> initializes each business
 * object with the <code>SettingsBean</code> immediately after it creates it.
 * 
 * @author David Tobey
 */
public class SettingsBean extends BaseBusinessProcessor implements Settings,
		Serializable {

	private static final long serialVersionUID = 4050487794147407160L;

	static Log log = LogFactory.getLog(SettingsBean.class);

	/**
	 * @see #getMap()
	 */
	private Map map = null;

	public Map getMap() {
		return (this.map);
	}

	public void setMap(Map map) {
		this.map = map;
	}

	public void setValue(String key, String value) {
		getMap().put(key, value);
	}

	public String getValue(String key) {
		return (String) getMap().get(key);
	}

	public void initialize() throws Exception {
		SettingsDAO settingsDAO = (SettingsDAO) getDaoFactory().createDAO(
				"settingsDAOImplementer");
		settingsDAO.loadSettings(this);
		overrideSettings();
	}

	public Settings copyAndAlter(String name, String value) throws Exception {
		Settings newSettings =  getBusinessObjectFactory()
				.createObject(Settings.class);
		newSettings.setMap(new HashMap());
		newSettings.getMap().putAll(this.getMap());
		newSettings.getMap().put(name, value);
		return newSettings;
	}

	public Settings copyAndAlter(Map settingsToAlter) throws Exception {
		Settings newSettings =  getBusinessObjectFactory()
				.createObject(Settings.class);
		newSettings.setMap(new HashMap());
		newSettings.getMap().putAll(this.getMap());
		newSettings.getMap().putAll(settingsToAlter);
		log.info("newSettings map: " + newSettings.getValue("trackInventory"));
		return newSettings;
	}

	public Settings processSettings(Map parameters) throws Exception {
		if (log.isDebugEnabled())
			log.debug("About to process settings.");
		Setting setting = (Setting) getBusinessObjectFactory().createObject(
				"settingImplementer");
		Settings settings = (Settings) getBusinessObjectFactory().createObject(
				"settingsImplementer");
		SettingsDAO settingsDAO = (SettingsDAO) getDaoFactory().createDAO(
				"settingsDAOImplementer");
		Settings s = settingsDAO.updateSettings(parameters, settings, setting);
		overrideSettings(s);
		return s;
	}

	public Collection loadSettings() throws Exception {
		if (log.isDebugEnabled())
			log.debug("About to load settings.");
		SettingsDAO settingsDAO = (SettingsDAO) getDaoFactory().createDAO(
				"settingsDAOImplementer");
		return settingsDAO.loadSettings();
	}

	public Map loadSettingsAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading settings and count");
		SettingsDAO gatewayDAO = (SettingsDAO) getDaoFactory().createDAO(
				"settingsDAOImplementer");
		return gatewayDAO.loadSettingsAndCount(parameters);
	}

	public Map addSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Adding Setting");
		Setting setting =  getBusinessObjectFactory().createObject(Setting.class);
		Map results = postChangingSettingEvent(setting, parameters);
		BeanUtils.copyProperties(setting, parameters);
		SettingsDAO settingDAO =  getDaoFactory().createDAO(SettingsDAO.class);
		settingDAO.setSetting(setting);
		settingDAO.insertSetting(true);
		results.put("newRecord", settingDAO.getSetting());
		return results;
	}

	public Setting loadSettingFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading setting from id");
		Setting setting = (Setting) getBusinessObjectFactory().createObject(
				"settingImplementer");
		SettingsDAO dao = (SettingsDAO) getDaoFactory().createDAO(
				"settingsDAOImplementer");
		BeanUtils.copyProperties(setting, parameters);
		dao.setSetting(setting);
		dao.loadSettingFromID();
		return dao.getSetting();
	}

	public Map editSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing Setting");
		Setting setting = loadSettingFromID(parameters);
		Map results = postChangingSettingEvent(setting, parameters);
		BeanUtils.copyProperties(setting, parameters);
		SettingsDAO settingDAO =  getDaoFactory().createDAO(SettingsDAO.class);
		settingDAO.setSetting(setting);
		settingDAO.updateSetting(true);
		results.put("updatedRecord", settingDAO.getSetting());
		return results;
	}

	public Map postChangingSettingEvent(Setting setting, Map parameters) throws Exception {
		Map results = new HashMap();
		if (getBusinessObjectFactory() != null && getEventBus() != null) {
			ChangingSettingEvent event = (ChangingSettingEvent) getBusinessObjectFactory().createObject(
					"changingSettingEventImplementer");
			event.setBusinessProcessor(this);
			event.setSetting(setting);
			event.setIncomingParameters(parameters);
			event.setResults(results);
			getEventBus().post(event);
			results = event.getResults();
		}
		return results;
	}

	public void deleteSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting setting: " + code);
		}
		Setting setting = (Setting) getBusinessObjectFactory().createObject(
				"settingImplementer");
		BeanUtils.copyProperties(setting, parameters);
		SettingsDAO dao = (SettingsDAO) getDaoFactory().createDAO(
				"settingsDAOImplementer");
		dao.setSetting(setting);
		dao.deleteSetting(true);
		return;
	}

	public Map editDeleteUserSettings(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple administrators");
		SettingsDAO dao =  getDaoFactory().createDAO(SettingsDAO.class);
		Map results = dao.editDeleteUserSettings(parameters, getBusinessObjectFactory());
		return results;
	}

	public void overrideSettings(Settings s) throws Exception {
		overrideSettings(s.getMap());
	}

	public void overrideSettings() throws Exception {
		overrideSettings(getMap());
	}

	public void overrideSettings(Map map) throws Exception {
		if (getAppSettings() != null) {
			Enumeration<?> keys = getAppSettings().propertyNames();
			// loop through all properties
			while (keys.hasMoreElements()) {
				String key = (String) keys.nextElement();
				if (map.containsKey(key)) {
					map.put(key, getAppSettings().getProperty(key));
				}
			}
		}
	}
}
