package com.softslate.commerce.businessobjects.order;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;
import com.softslate.commerce.businessobjects.product.Discount;
import com.softslate.commerce.businessobjects.product.Product;

/**
 * Interface for business logic methods having to do with discounts in a user's
 * cart.
 * <p>
 * {@link com.softslate.commerce.businessobjects.order.CartProcessor} employs an
 * instance implementing <code>CartDiscountProcessor</code> when discounting
 * needs to be processed.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CartDiscountProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "cartDiscountProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "cartDiscountProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor}.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.order.InventoryProcessor
 */
public interface CartDiscountProcessor extends BusinessProcessor {

	/**
	 * Processes discounts upon adding an item or items to the user's cart. The
	 * default implementation simply calls
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor#processDiscounts(Collection)}
	 * 
	 * @param orderItems
	 *            The collection of <code>OrderItem</code> objects being added
	 *            to the cart.
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processAddItems(Collection orderItems) throws Exception;

	/**
	 * Processes discounts during checkout, as taxes and shipping are processed.
	 * The default implementation simply calls
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor#processDiscounts(Collection)}
	 * 
	 * @param parameters
	 *            A <code>Map</code> with any parameters needed for the
	 *            processing. The default implementation does not use this
	 *            argument.
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processOnCheckout(Map parameters) throws Exception;

	/**
	 * Processes discounts after shipping and taxes have been calculated. This
	 * is so that if there are any gift certificates, they may be applied
	 * against the order's total rather than its subtotal as is the case with
	 * regular discounts. The default implementation simply calls
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor#processDiscounts(Collection)}
	 * 
	 * @param parameters
	 *            A <code>Map</code> with any parameters needed for the
	 *            processing. The default implementation does not use this
	 *            argument.
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processAfterTaxes(Map parameters) throws Exception;

	/**
	 * Processes discounts as a user enters a coupon code. The default
	 * implementation simply calls
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor#processDiscounts(Collection)}
	 * 
	 * @param parameters
	 *            A <code>Map</code> with any parameters needed for the
	 *            processing. The default implementation does not use this
	 *            argument.
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processCouponCode(Map parameters) throws Exception;

	/**
	 * Processes discounts as a customer logs in. Some discounts only apply if
	 * the user is logged in as a customer, so discounts must be rerun. The
	 * default implementation simply calls
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor#processDiscounts(Collection)}
	 * 
	 * @param parameters
	 *            A <code>Map</code> with any parameters needed for the
	 *            processing. The default implementation does not use this
	 *            argument.
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processCustomerLogin(Map parameters) throws Exception;

	/**
	 * Processes discounts as a customer logs out. Some discounts only apply if
	 * the user is logged in as a customer, so discounts must be rerun. The
	 * default implementation simply calls
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor#processDiscounts(Collection)}
	 * 
	 * @param parameters
	 *            A <code>Map</code> with any parameters needed for the
	 *            processing. The default implementation does not use this
	 *            argument.
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processCustomerLogout(Map parameters) throws Exception;

	/**
	 * Processes discounts as a user's order is completed. The default
	 * implementation simply calls
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartDiscountProcessor#processDiscounts(Collection)}
	 * 
	 * @param parameters
	 *            A <code>Map</code> with any parameters needed for the
	 *            processing. The default implementation does not use this
	 *            argument.
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processOrderComplete(Map parameters) throws Exception;

	/**
	 * Removes all the discounts in the current user's cart. Updates the cart
	 * totals accordingly. Called before items are removed from the cart by
	 * {@link com.softslate.commerce.businessobjects.order.BasicCartProcessor#processRemoveItem(OrderItem)}
	 * 
	 * @return A <code>Map</code> containing results of the processing. The
	 *         default implementation returns null.
	 * @throws Exception
	 */
	Map processRemoveDiscounts() throws Exception;

	/**
	 * Checks that a given discount is applicable. Used by discount processing
	 * to help determine which discounts to apply, and in the presentation to
	 * determine which discounts to display to the user.
	 * 
	 * @param discount
	 *            The discount whose applicability is being checked.
	 * @param previousOrderTotal
	 *            The previous order total of the current user, or null. Some
	 *            discounts are only applicable if the user has previously
	 *            placed orders totaling a certain amount.
	 * @return true, the discount is applicable, otherwise, false.
	 * @throws Exception
	 */
	boolean isDiscountOk(Discount discount, Double previousOrderTotal)
			throws Exception;

	/**
	 * Checks if a given discount would be applicable, if the given product code
	 * were in the user's cart. Used by the presentation layer to communicate
	 * the discounts that would be available if the item were added to the cart.
	 * <p>
	 * 
	 * @param discount
	 *            The discount whose applicability is being checked.
	 * @param previousOrderTotal
	 *            The previous order total of the current user, or null. Some
	 *            discount are only applicable if the user has previously placed
	 *            orders totaling a certain amount.
	 * @return true, the discount is applicable, otherwise, false.
	 * @throws Exception
	 */
	boolean isDiscountOkIfItemInCart(Discount discount,
			Double previousOrderTotal, String productCode) throws Exception;

	/**
	 * Applies shipping discounts to each of the shipping options in the incoming
	 * Collection.
	 * 
	 * @param shippingOptions
	 * @return the same Collection of shipping options, with discounts applied
	 * @throws Exception
	 */
	public Collection processShippingDiscountsOnShippingOptions(
			Collection shippingOptions) throws Exception;

	/**
	 * Inspects a Product and returns a Map of useful variables related to the
	 * state of the product's discounts.
	 * 
	 * @param product
	 * @param attributeSKUCollection
	 *            An optional Collection of attribute-only SKUs used to define
	 *            the inventory settings that are returned.
	 * @param storeDiscounts
	 *            An optional Collection of global, store discounts, used to
	 *            determine if "promotions" for the product exist, ie, discounts
	 *            that would be applied if the product were added to the cart.
	 * @return A Map of variables related to the product's discounts.
	 * @throws Exception
	 */
	public Map defineDiscountSettings(Product product,
			Collection attributeSKUCollection, Collection storeDiscounts)
			throws Exception;
}
