package com.softslate.commerce.businessobjects.product;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.axis.utils.StringUtils;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.events.ChangingProductEvent;
import com.softslate.commerce.daos.product.AttributeDAO;
import com.softslate.commerce.daos.product.CategoryDAO;
import com.softslate.commerce.daos.product.ProductDAO;
import com.softslate.commerce.daos.product.ProductGatewayDAO;

/**
 * Processes business logic affecting products and other related objects.
 * <code>BasicProductProcessor</code> is the default implementation of the
 * <code>ProductProcessor</code> interface for the application.
 * 
 * @author David Tobey
 */
public class BasicProductProcessor extends BaseBusinessProcessor implements ProductProcessor {

	static Log log = LogFactory.getLog(BasicProductProcessor.class);

	public Collection loadCategoryTree() throws Exception {
		ProductGatewayDAO productGatewayDAO = (ProductGatewayDAO) getDaoFactory().createDAO(
				"productGatewayDAOImplementer");
		return productGatewayDAO.loadCategoryTree();
	}

	public Map createCategoryMapFromCategoryTree(Collection categoryTree) {
		Map categoryMap = new HashMap();
		appendToCategoryMap(categoryMap, categoryTree);
		List<Entry<String, Category>> categorylist = new LinkedList<Entry<String, Category>>(categoryMap.entrySet());
		Collections.sort(categorylist, new Comparator<Entry<String, Category>>() {
			@Override
			public int compare(Entry<String, Category> category1, Entry<String, Category> category2) {
				Integer order1 = category1.getValue().getCategoryOrder();
				Integer order2 = category2.getValue().getCategoryOrder();
				return order1.compareTo(order2);
			}
		});
		Map<String, Category> sortedcategorymap = new LinkedHashMap<String, Category>();
		for (Entry<String, Category> entry : categorylist) {
			sortedcategorymap.put(entry.getKey(), entry.getValue());
		}
		return sortedcategorymap;
	}

	protected void appendToCategoryMap(Map categoryMap, Collection categories) {
		if (categories != null) {
			Iterator i = categories.iterator();
			while (i.hasNext()) {
				Category category = (Category) i.next();
				categoryMap.put(category.getCode(), category);
				appendToCategoryMap(categoryMap, category.getSubcategoryCollection());
			}
		}
	}

	public Collection loadBuiltInCategories() throws Exception {
		Collection categories = new ArrayList();
		String builtInCategories = (String) getSettings().getValue("builtInCategories");
		if (builtInCategories != null && builtInCategories.length() > 0) {
			String[] cats = builtInCategories.split(",");
			for (int i = 0; i < cats.length; i++) {
				Category cat =  getBusinessObjectFactory().createObject(Category.class);
				cat.setCode(cats[i]);
				cat.setItemsPerPage(Integer.parseInt((String) getSettings().getValue("builtInCategoryProductCount")));
				cat = getCategoryFromCode(cat);
				if (cat.getCategoryID() != 0) {
					categories.add(cat);
				}
			}
		}
		return categories;
	}

	public Collection loadCategories(String categoryList) throws Exception {
		Collection categories = new ArrayList();
		if (categoryList == null || categoryList.length() == 0) {
			return categories;
		}
		String[] cats = categoryList.split(",");
		for (int i = 0; i < cats.length; i++) {
			Category cat =  getBusinessObjectFactory().createObject(Category.class);
			cat.setCode(cats[i]);
			cat.setItemsPerPage(0);
			cat = getCategoryFromCode(cat);
			if (cat.getCategoryID() != 0) {
				categories.add(cat);
			}
		}
		return categories;
	}

	public Product getProductFromCode(Product product) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading product: " + product.getCode());
		ProductDAO productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		// Load the product and its attributes and options.
		productDAO.setProduct(product);
		productDAO.loadProduct();
		if (productDAO.getProduct() == null) {
			return product;
		}
		product = productDAO.getProduct();
		product.addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
		return product;
	}

	public Product getProductFromSeoCode(Product product) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading product from SEO code: " + product.getSeoCode());
		ProductDAO productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		// Load the product and its attributes and options.
		productDAO.setProduct(product);
		productDAO.loadProductFromSeoCode();
		if (productDAO.getProduct() == null) {
			return product;
		}
		product = productDAO.getProduct();
		product.addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
		return product;
	}

	public Map loadAttributeSKUs() throws Exception {
		if (log.isDebugEnabled())
			log.debug("Loading all attribute SKUs.");
		ProductGatewayDAO productGatewayDAO = (ProductGatewayDAO) getDaoFactory().createDAO(
				"productGatewayDAOImplementer");
		Collection skus = productGatewayDAO.loadAttributeSKUs();
		ProductBean product = new ProductBean();
		return product.buildSkuMap(skus);
	}

	public Collection loadAttributeSKUCollection() throws Exception {
		if (log.isDebugEnabled())
			log.debug("Loading all attribute SKUs.");
		ProductGatewayDAO productGatewayDAO = (ProductGatewayDAO) getDaoFactory().createDAO(
				"productGatewayDAOImplementer");
		return productGatewayDAO.loadAttributeSKUs();
	}

	public Category getCategoryFromCode(Category category) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading category: " + category.getCode());
		return getCategoryFromCode(category, true);
	}

	public Category getCategoryFromCode(Category category, boolean loadProducts) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading category: " + category.getCode());
		CategoryDAO categoryDAO =  getDaoFactory().createDAO(CategoryDAO.class);
		categoryDAO.setCategory(category);
		// Load the category itself, preserving the original paging params.
		categoryDAO.loadCategory();
		if (categoryDAO.getCategory() == null) {
			return category;
		}
		// Preserve the pagination parameters
		categoryDAO.getCategory().setItemsPerPage(category.getItemsPerPage());
		categoryDAO.getCategory().setFirstRow(category.getFirstRow());
		category = categoryDAO.getCategory();
		// Initialize the category's parents.
		if (log.isDebugEnabled())
			log.debug("Loading parents: " + category.getCode());
		categoryDAO.loadParents();
		// Initialize the subcategory collection and settings
		category.getSubcategoryCollection();
		category.getCategorySettings();
		// Load the category's products.
		if (loadProducts) {
			if (log.isDebugEnabled())
				log.debug("Loading category's product list: " + category.getCode());
			ProductList productList =  getBusinessObjectFactory().createObject(ProductList.class);
			productList.setProductListType(ProductListBean.CATEGORY);
			productList.setCategoryID(category.getCategoryID());
			productList.setItemsPerPage(category.getItemsPerPage());
			productList.setFirstRow(category.getFirstRow());
			productList.setItemCount(category.getItemCount());
			ProductGatewayDAO productGatewayDAO = (ProductGatewayDAO) getDaoFactory().createDAO(
					"productGatewayDAOImplementer");
			productGatewayDAO.loadProductList(productList);
			productList.addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
			category.setProductListCollection(productList.getProductListCollection());
			category.setItemCount(productList.getItemCount());
		}
		return category;
	}

	public HashMap generateSettingsMap(Collection settings) throws Exception {
		Iterator i = settings.iterator();
		HashMap map = new HashMap();
		while (i.hasNext()) {
			ProductSetting setting = (ProductSetting) i.next();
			String settingCode = setting.getCode();
			String valueType = setting.getValueType();
			String settingValue = null;
			if (valueType.equals("smallValue")) {
				settingValue = setting.getSmallValue();
			} else if (valueType.equals("mediumValue")) {
				settingValue = setting.getMediumValue();
			} else if (valueType.equals("textValue")) {
				settingValue = setting.getTextValue();
			} else if (valueType.equals("integerValue")) {
				settingValue = Integer.toString(setting.getIntegerValue());
			}
			if (settingValue == null) {
				settingValue = "";
			}
			map.put(settingCode, settingValue);
		}
		return map;
	}

	public Collection getAllAttributesAndOptions(int productID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading attrs and options for product: " + productID);
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		product.setProductID(productID);
		ProductDAO productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		productDAO.setProduct(product);
		productDAO.loadAttributesAndOptions();
		return product.getAttributesAndOptions();
	}

	public ProductList getProductList(ProductList productList) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading product list");
		ProductGatewayDAO productGatewayDAO = (ProductGatewayDAO) getDaoFactory().createDAO(
				"productGatewayDAOImplementer");
		productGatewayDAO.loadProductList(productList);
		productList.addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
		return productList;
	}

	public Collection loadAllProducts() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading all products");
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadAllProducts();
	}

	public int loadMaxProductID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName() + ": Loading max product ID.");
		}
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadMaxProductID();
	}

	public Collection loadProductsForIndexing(int firstProductID, int lastProductID) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName() + ": Loading products for indexing.");
			log.info("firstProductID: " + firstProductID);
			log.info("lastProductID: " + lastProductID);
		}
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadProductsForIndexing(firstProductID, lastProductID);
	}

	public Map sortProductsByName(Map parameters) throws Exception {

		// Sort them
		Map results = new HashMap();
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		Collection products = loadAllProductsSortedByName();
		int sortOrder = 0;
		Iterator i = products.iterator();
		while (i.hasNext()) {
			Integer productID = (Integer) i.next();
			sortOrder++;
			if (log.isDebugEnabled())
				log.debug("Updating productOrder to " + sortOrder + " where productID is " + productID);
			dao.updateProductOrderByID(productID.intValue(), sortOrder);
		}

		// Update the orders in the linking tables
		String keepOrdersInSync = (String) getSettings().getValue("keepOrdersInSync");
		if (keepOrdersInSync != null && keepOrdersInSync.equals("1")) {
			ProductGatewayDAO gatewayDao =  getDaoFactory()
					.createDAO(ProductGatewayDAO.class);
			gatewayDao.updateProductCategoryOrders();
			gatewayDao.updateRelatedProductOrders();
		}

		results.put("productsSorted", new Integer(sortOrder));
		return results;
	}

	public Collection loadAllProductsSortedByName() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading all products sorted by name");
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadAllProductsSortedByName();
	}

	public Map sortProductsByCode(Map parameters) throws Exception {

		// Sort them
		Map results = new HashMap();
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		Collection products = loadAllProductsSortedByCode();
		int sortOrder = 0;
		Iterator i = products.iterator();
		while (i.hasNext()) {
			Integer productID = (Integer) i.next();
			sortOrder++;
			if (log.isDebugEnabled())
				log.debug("Updating productOrder to " + sortOrder + " where productID is " + productID);
			dao.updateProductOrderByID(productID.intValue(), sortOrder);
		}

		// Update the orders in the linking tables
		String keepOrdersInSync = (String) getSettings().getValue("keepOrdersInSync");
		if (keepOrdersInSync != null && keepOrdersInSync.equals("1")) {
			ProductGatewayDAO gatewayDao =  getDaoFactory()
					.createDAO(ProductGatewayDAO.class);
			gatewayDao.updateProductCategoryOrders();
			gatewayDao.updateRelatedProductOrders();
		}

		results.put("productsSorted", new Integer(sortOrder));
		return results;
	}

	public Collection loadAllProductsSortedByCode() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading all products sorted by code");
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadAllProductsSortedByCode();
	}

	public Collection loadAllProductNamesCodesIDs() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading all products");
		int maxProducts = 1000;
		String maxAdminRecordsToDisplay = (String) getSettings().getValue("maxAdminRecordsToDisplay");
		if (maxAdminRecordsToDisplay != null) {
			maxProducts = Integer.parseInt(maxAdminRecordsToDisplay);
		}
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		Long count = gatewayDAO.loadProductCount();
		if (count.longValue() > maxProducts) {
			return null;
		}
		return gatewayDAO.loadAllProductNamesCodesIDs();
	}

	public Collection loadActiveProductNamesCodesIDs() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading active products");
		int maxProducts = 1000;
		String maxAdminRecordsToDisplay = (String) getSettings().getValue("maxAdminRecordsToDisplay");
		if (maxAdminRecordsToDisplay != null) {
			maxProducts = Integer.parseInt(maxAdminRecordsToDisplay);
		}
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		Long count = gatewayDAO.loadProductCount();
		if (count.longValue() > maxProducts) {
			return null;
		}
		return gatewayDAO.loadActiveProductNamesCodesIDs();
	}

	public Map loadProductsAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading products and count");
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return gatewayDAO.loadProductsAndCount(parameters);
	}

	public Map addProduct(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Adding product: " + code);
		}
		Map results = new HashMap();

		// Check that another product with the same code doesn't exist
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		BeanUtils.copyProperties(product, parameters);
		results = validateProduct(product);
		if (results != null && results.get("resultCode") != null && !results.get("resultCode").equals("0")) {
			return results;
		}

		// Insert the product
		if (product.getPrimaryCategoryID() != null && product.getPrimaryCategoryID().intValue() != 0) {
			Category category =  getBusinessObjectFactory().createObject(Category.class);
			category.setCategoryID(product.getPrimaryCategoryID().intValue());
			product.setPrimaryCategory(category);
		}
		if (product.getManufacturerID() != null && product.getManufacturerID().intValue() != 0) {
			Manufacturer manufacturer = (Manufacturer) getBusinessObjectFactory().createObject(
					"manufacturerImplementer");
			manufacturer.setManufacturerID(product.getManufacturerID().intValue());
			product.setManufacturer(manufacturer);
		}
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		dao.setProduct(product);
		dao.insertProduct(true);
		results.put("newRecord", dao.getProduct());

		// Now attempt to insert a SKU corresponding to the product
		try {
			SKUProcessor processor =  getBusinessObjectFactory().createObject(SKUProcessor.class);
			Map skuParameters = new HashMap();
			skuParameters.put("productID", Integer.toString(product.getProductID()));
			skuParameters.put("skuNumber", product.getCode());
			skuParameters.put("skuName", product.getName());
			skuParameters.put("trackInventory", "false");
			Map skuResults = processor.addSKU(skuParameters);
			results.put("skuResults", skuResults);
		} catch (Exception e) {
			// Swallow
		}

		return results;
	}

	public Product loadProductFromID(Map parameters) throws Exception {
		int productID = 0;
		if (parameters.get("productID") instanceof String) {
			productID = new Integer((String) parameters.get("productID"));
		} else if (parameters.get("productID") instanceof Integer) {
			productID = (Integer) parameters.get("productID");
		}
		return this.loadProductFromID(productID);
	}

	public Product loadProductFromID(int productID) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading product from id");
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		product.setProductID(productID);
		dao.setProduct(product);
		dao.loadProductFromID();
		return dao.getProduct();
	}

	public Product loadProductFromCode(Map parameters) throws Exception {
		return this.loadProductFromCode((String) parameters.get("code"));
	}

	public Product loadProductFromCode(String code) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading product from code");
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		product.setCode(code);
		dao.setProduct(product);
		dao.loadProductFromCode();
		return dao.getProduct();
	}

	public Map editProduct(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Editing product: " + code);
		}
		Map results = new HashMap();

		// Check that another product with the same code doesn't exist
		if (parameters.get("productID") != null) {
			Product product =  getBusinessObjectFactory().createObject(Product.class);
			BeanUtils.copyProperties(product, parameters);
			results = validateProduct(product);
			if (results != null && results.get("resultCode") != null && !results.get("resultCode").equals("0")) {
				return results;
			}
		}

		// Load the product
		Product product = null;
		if (parameters.get("productID") != null) {
			product = loadProductFromID(parameters);
		}
		if (product == null && parameters.get("code") != null) {
			product = loadProductFromCode(parameters);
		}

		results = postChangingProductEvent(product, parameters);

		// Update the product
		BeanUtils.copyProperties(product, parameters);
		if (parameters.get("primaryCategoryID") != null && !parameters.get("primaryCategoryID").equals("")
				&& !parameters.get("primaryCategoryID").equals("0")) {
			Category category =  getBusinessObjectFactory().createObject(Category.class);
			category.setCategoryID(product.getPrimaryCategoryID().intValue());
			product.setPrimaryCategory(category);
		} else if (parameters.get("primaryCategoryID") != null
				&& (parameters.get("primaryCategoryID").equals("") || parameters.get("primaryCategoryID").equals("0"))) {
			product.setPrimaryCategory(null);
		}
		if (parameters.get("manufacturerID") != null && !parameters.get("manufacturerID").equals("")
				&& !parameters.get("manufacturerID").equals("0")) {
			Manufacturer manufacturer = (Manufacturer) getBusinessObjectFactory().createObject(
					"manufacturerImplementer");
			manufacturer.setManufacturerID(product.getManufacturerID().intValue());
			product.setManufacturer(manufacturer);
		} else if (parameters.get("manufacturerID") != null
				&& (parameters.get("manufacturerID").equals("") || parameters.get("manufacturerID").equals("0"))) {
			product.setManufacturer(null);
		}
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		dao.setProduct(product);
		dao.updateProduct(true);
		results.put("updatedRecord", dao.getProduct());
		String keepOrdersInSync = (String) getSettings().getValue("keepOrdersInSync");
		if (keepOrdersInSync != null && keepOrdersInSync.equals("1")) {
			ProductGatewayDAO gatewayDao =  getDaoFactory()
					.createDAO(ProductGatewayDAO.class);
			gatewayDao.updateProductCategoryOrders();
			gatewayDao.updateRelatedProductOrders();
		}
		return results;
	}

	public Map postChangingProductEvent(Product product, Map parameters) throws Exception {
		Map results = new HashMap();
		ChangingProductEvent event = getBusinessObjectFactory().createObject(ChangingProductEvent.class);
		event.setBusinessProcessor(this);
		event.setProduct(product);
		event.setIncomingParameters(parameters);
		event.setResults(results);
		getEventBus().post(event);
		return event.getResults();
	}

	public Map editDeleteProducts(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple products");
		String[] idValues = (String[]) parameters.get("idValues");
		String[] displayFields = (String[]) parameters.get("displayFields");
		Category[] primaryCategory = new Category[idValues.length];
		Manufacturer[] manufacturer = new Manufacturer[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			primaryCategory[i] = null;
			manufacturer[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("primaryCategoryID") || displayField.equals("primaryCategory")) {
					displayFields[j] = "primaryCategory";
					String[] fieldValues = (String[]) parameters.get("primaryCategoryID");
					if (fieldValues != null && fieldValues[i] != null && !fieldValues[i].equals("")
							&& !fieldValues[i].equals("0")) {
						if (log.isDebugEnabled())
							log.debug("Creating primary category of " + fieldValues[i] + " for id: " + idValues[i]);
						Category object =  getBusinessObjectFactory().createObject(Category.class);
						object.setCategoryID((new Integer(fieldValues[i])).intValue());
						primaryCategory[i] = object;
					}
				}
				if (displayField.equals("manufacturerID") || displayField.equals("manufacturer")) {
					displayFields[j] = "manufacturer";
					String[] fieldValues = (String[]) parameters.get("manufacturerID");
					if (fieldValues != null && fieldValues[i] != null && !fieldValues[i].equals("")
							&& !fieldValues[i].equals("0")) {
						if (log.isDebugEnabled())
							log.debug("Creating manufacturer " + fieldValues[i] + " for id: " + idValues[i]);
						Manufacturer object = (Manufacturer) getBusinessObjectFactory().createObject(
								"manufacturerImplementer");
						object.setManufacturerID((new Integer(fieldValues[i])).intValue());
						manufacturer[i] = object;
					}
				}
			}
		}
		parameters.put("primaryCategory", primaryCategory);
		parameters.put("manufacturer", manufacturer);
		ProductGatewayDAO dao =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		Map results = dao.editDeleteProducts(parameters, getBusinessObjectFactory());
		String keepOrdersInSync = (String) getSettings().getValue("keepOrdersInSync");
		if (keepOrdersInSync != null && keepOrdersInSync.equals("1")) {
			dao.updateProductCategoryOrders();
			dao.updateRelatedProductOrders();
		}
		return results;
	}

	public Map deleteProduct(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Deleting product: " + code);
		}
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		BeanUtils.copyProperties(product, parameters);
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		dao.setProduct(product);
		dao.deleteProduct(true);
		return null;
	}

	public Map validateProduct(Product product) throws Exception {
		Map results = new HashMap();

		// Check for a duplicate code
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		dao.setProduct(product);
		dao.loadProductFromCode();
		if (dao.getProduct() != null) {
			dao.getDaoFactory().getSession().evict(dao.getProduct());
		}
		boolean insertingButExists = product.getProductID() == 0 && dao.getProduct() != null
				&& dao.getProduct().getProductID() > 0;
		boolean updatingButExists = product.getProductID() > 0 && dao.getProduct() != null
				&& product.getProductID() != dao.getProduct().getProductID();
		if (insertingButExists || updatingButExists) {
			results.put("resultCode", "DuplicateRecord");
			results.put("duplicateField", "code");
			results.put("existingRecord", dao.getProduct());
		}

		// Check for a duplicate seoCode
		if (product.getSeoCode() != null && product.getSeoCode().length() > 0) {
			dao =  getDaoFactory().createDAO(ProductDAO.class);
			dao.setProduct(product);
			dao.loadProductFromSeoCodeAdmin();
			if (dao.getProduct() != null) {
				dao.getDaoFactory().getSession().evict(dao.getProduct());
			}
			insertingButExists = product.getProductID() == 0 && dao.getProduct() != null
					&& dao.getProduct().getProductID() > 0;
			updatingButExists = product.getProductID() > 0 && dao.getProduct() != null
					&& product.getProductID() != dao.getProduct().getProductID();
			if (insertingButExists || updatingButExists) {
				results.put("resultCode", "DuplicateRecord");
				results.put("duplicateField", "seoCode");
				results.put("existingRecord", dao.getProduct());
			}
		}
		return results;
	}

	public Collection loadSettings(Map parameters) throws Exception {
		if (log.isDebugEnabled())
			log.debug("About to load product settings.");
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		BeanUtils.copyProperties(product, parameters);
		ProductDAO dao =  getDaoFactory().createDAO(ProductDAO.class);
		dao.setProduct(product);
		return dao.loadSettings();
	}

	public Collection processSettings(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("About to process product settings.");
		// Load the settings
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		BeanUtils.copyProperties(product, parameters);
		ProductDAO productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		productDAO.setProduct(product);
		Collection settings = productDAO.loadSettings();
		parameters.put("settings", settings);
		parameters.put("productDAO", productDAO);
		if (log.isDebugEnabled())
			log.debug("Determining new product settings to insert.");
		// Create a collection of product settings that are new and must be
		// inserted.
		Collection productSettingsToInsert = new ArrayList();
		String[] codes = (String[]) parameters.get("code");
		String[] names = (String[]) parameters.get("name");
		String[] descriptions = (String[]) parameters.get("description");
		String[] integerValue = (String[]) parameters.get("integerValue");
		String[] smallValue = (String[]) parameters.get("smallValue");
		String[] mediumValue = (String[]) parameters.get("mediumValue");
		String[] textValue = (String[]) parameters.get("textValue");
		String[] values = (String[]) parameters.get("value");
		String[] valueTypes = (String[]) parameters.get("valueType");
		for (int i = 0; i < codes.length; i++) {
			boolean insert = true;
			Iterator j = settings.iterator();
			while (j.hasNext()) {
				ProductSetting setting = (ProductSetting) j.next();
				if (setting.getCode().equals(codes[i])) {
					if (log.isDebugEnabled())
						log.debug("Existing product setting: " + codes[i]);
					setting.setProduct(product);
					insert = false;
				}
			}
			if (insert) {
				if (log.isDebugEnabled())
					log.debug("New product setting: " + codes[i]);
				ProductSetting productSetting = (ProductSetting) getBusinessObjectFactory().createObject(
						"productSettingImplementer");
				productSetting.setProductID(Integer.parseInt((String) parameters.get("productID")));
				productSetting.setCode(codes[i]);
				productSetting.setName(names[i]);
				productSetting.setDescription(descriptions[i]);
				productSetting.setValueType(valueTypes[i]);
				productSetting.setType((String) parameters.get("type"));
				if (integerValue != null && !StringUtils.isEmpty(integerValue[i])) {
					try {
						productSetting.setIntegerValue(Integer.parseInt(integerValue[i]));
					} catch (NumberFormatException nfe) {
						// swallow
					}
				}
				if (smallValue != null) {
					productSetting.setSmallValue(smallValue[i]);
				}
				if (mediumValue != null) {
					productSetting.setMediumValue(mediumValue[i]);
				}
				if (textValue != null) {
					productSetting.setTextValue(textValue[i]);
				}
				if (valueTypes[i].equals("integerValue") || valueTypes[i].equals("booleanValue")) {
					productSetting.setIntegerValue(Integer.parseInt(values[i]));
				} else if (valueTypes[i].equals("smallValue")) {
					productSetting.setSmallValue(values[i]);
				} else if (valueTypes[i].equals("mediumValue")) {
					productSetting.setMediumValue(values[i]);
				} else if (valueTypes[i].equals("textValue")) {
					productSetting.setTextValue(values[i]);
				}
				productSetting.setProduct(product);
				productSettingsToInsert.add(productSetting);
			}
		}
		parameters.put("productSettingsToInsert", productSettingsToInsert);
		// Process the updates and inserts in the database
		ProductGatewayDAO dao =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		return dao.processSettings(true, parameters);
	}

	public Collection buildMatrix(Product product) {
		if (log.isDebugEnabled())
			log.debug("Starting matrix.");
		Collection matrix = new ArrayList();
		Collection usedAttrs = new ArrayList();
		Collection row = new ArrayList();
		buildMatrix(matrix, row, usedAttrs, product);
		if (log.isDebugEnabled())
			log.debug("Done with matrix.");
		return matrix;
	}

	public boolean buildMatrix(Collection matrix, Collection options, Collection attributes, Product product) {
		if (log.isDebugEnabled())
			log.debug("Starting buildMatrix.");
		boolean moreAttributes = false;
		Collection pas = product.getProductAttributes();
		Iterator pai = pas.iterator();
		while (pai.hasNext()) {
			ProductAttribute pa = (ProductAttribute) pai.next();
			Attribute a = (Attribute) pa.getAttribute();
			if (log.isDebugEnabled())
				log.debug("Checking attribute: " + a.getAttributeCode());
			if (!a.getAttributeIsActive() || attributes.contains(a) || a.getAttributeOptions() == null
					|| a.getAttributeOptions().isEmpty()) {
				continue;
			}
			if (log.isDebugEnabled())
				log.debug("Procesing attribute: " + a.getAttributeCode());
			moreAttributes = true;
			attributes.add(a);
			Collection os = a.getAttributeOptions();
			Iterator osi = os.iterator();
			while (osi.hasNext()) {
				Option o = (Option) osi.next();
				if (log.isDebugEnabled())
					log.debug("Procesing option: " + o.getOptionCode());
				if (o.getOptionIsActive()) {
					options.add(o);
					if (!buildMatrix(matrix, options, attributes, product)) {
						if (log.isDebugEnabled())
							log.debug("No more attributes to process. Adding row.");
						Collection copyRow = new ArrayList();
						// Add options to the row first
						Iterator rowI = options.iterator();
						while (rowI.hasNext()) {
							Option copyO = (Option) rowI.next();
							if (log.isDebugEnabled())
								log.debug("Adding option to row: " + copyO.getOptionCode());
							copyRow.add(copyO);
						}
						matrix.add(copyRow);
					}
					if (log.isDebugEnabled())
						log.debug("Removing option: " + o.getOptionCode());
					options.remove(o);
				}
			}
			if (log.isDebugEnabled())
				log.debug("Removing attribute: " + a.getAttributeCode());
			attributes.remove(a);
			// Since all the attributes are handled recursively, stop the
			// loop if this is the outside loop
			if (attributes.isEmpty()) {
				break;
			}
		}
		if (log.isDebugEnabled())
			log.debug("Ending buildMatrix.");
		return moreAttributes;
	}

	public Map cloneProduct(Map parameters) throws Exception {
		if (log.isInfoEnabled()) {
			log.info("Cloning product: " + parameters.get("productID"));
		}

		// Query for the source product
		Product product =  getBusinessObjectFactory().createObject(Product.class);
		product.setProductID(Integer.parseInt((String) parameters.get("productID")));
		ProductDAO productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		productDAO.setProduct(product);

		// Load the product
		productDAO.loadProductFromID();

		// Duplicate it
		Product dupeProduct = duplicateProduct(productDAO.getProduct(), parameters);
		if (dupeProduct != null) {
			productDAO.setProduct(dupeProduct);
			productDAO.insertProduct(true);
		}

		Map results = new HashMap();
		results.put("clonedProduct", dupeProduct);
		return results;
	}

	public Product duplicateProduct(Product product, Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Duplicating product");
		String codePrefix = (String) parameters.get("codePrefix");
		String cloneAttributes = (String) parameters.get("cloneAttributes");
		String cloneCategories = (String) parameters.get("cloneCategories");
		String cloneRelatedProducts = (String) parameters.get("cloneRelatedProducts");
		if (product != null) {
			Product dupeProduct =  getBusinessObjectFactory().createObject(Product.class);
			BeanUtils.copyProperties(dupeProduct, product);
			dupeProduct.setProductID(0);
			dupeProduct.setCode(codePrefix + product.getCode());
			if (product.getSeoCode() != null) {
				dupeProduct.setSeoCode(codePrefix + product.getSeoCode());
			}
			dupeProduct.setIsActive(false);
			dupeProduct.setProductAttributes(null);
			dupeProduct.setProductCategories(null);
			dupeProduct.setProductSettings(null);
			dupeProduct.setRelatedProducts(null);
			dupeProduct.setRelatedProductsAdmin(null);
			dupeProduct.setProductReviews(null);
			dupeProduct.setSkus(null);
			dupeProduct.setCustomerWishListProducts(null);
			if (cloneAttributes.equals("assign")) {
				if (product.getProductAttributes() != null && !product.getProductAttributes().isEmpty()) {
					dupeProduct.setProductAttributes(new LinkedHashSet());
					Iterator pas = product.getProductAttributes().iterator();
					while (pas.hasNext()) {
						ProductAttribute pa = (ProductAttribute) pas.next();
						ProductAttribute dupePA = (ProductAttribute) getBusinessObjectFactory().createObject(
								"productAttributeImplementer");
						BeanUtils.copyProperties(dupePA, pa);
						dupePA.setProductAttributeID(0);
						dupePA.setProductID(0);
						dupePA.setProduct(dupeProduct);
						dupeProduct.getProductAttributes().add(dupePA);
					}
				}
			} else if (cloneAttributes.equals("copy")) {
				if (product.getProductAttributes() != null && !product.getProductAttributes().isEmpty()) {
					dupeProduct.setProductAttributes(new LinkedHashSet());
					Iterator pas = product.getProductAttributes().iterator();
					while (pas.hasNext()) {
						ProductAttribute pa = (ProductAttribute) pas.next();
						Attribute dupeAttribute = duplicateAttribute(pa.getAttribute(), parameters);
						ProductAttribute dupePA = (ProductAttribute) getBusinessObjectFactory().createObject(
								"productAttributeImplementer");
						BeanUtils.copyProperties(dupePA, pa);
						dupePA.setProductAttributeID(0);
						dupePA.setProductID(0);
						dupePA.setAttributeID(0);
						dupePA.setAttribute(dupeAttribute);
						dupePA.setProduct(dupeProduct);
						dupeProduct.getProductAttributes().add(dupePA);
					}
				}
			}
			if (cloneCategories.equals("assign")) {
				if (product.getProductCategories() != null && !product.getProductCategories().isEmpty()) {
					dupeProduct.setProductCategories(new LinkedHashSet());
					Iterator pcs = product.getProductCategories().iterator();
					while (pcs.hasNext()) {
						ProductCategory pc = (ProductCategory) pcs.next();
						ProductCategory dupePC = (ProductCategory) getBusinessObjectFactory().createObject(
								"productCategoryImplementer");
						BeanUtils.copyProperties(dupePC, pc);
						dupePC.setProductCategoryID(0);
						dupePC.setProductID(0);
						dupePC.setProduct(dupeProduct);
						dupeProduct.getProductCategories().add(dupePC);
					}
				}
			}
			if (cloneRelatedProducts.equals("assign")) {
				if (product.getRelatedProducts() != null && !product.getRelatedProducts().isEmpty()) {
					dupeProduct.setRelatedProducts(new LinkedHashSet());
					Iterator rps = product.getRelatedProducts().iterator();
					while (rps.hasNext()) {
						RelatedProduct rp = (RelatedProduct) rps.next();
						RelatedProduct dupeRP = (RelatedProduct) getBusinessObjectFactory().createObject(
								"relatedProductImplementer");
						BeanUtils.copyProperties(dupeRP, rp);
						dupeRP.setRelatedProductID(0);
						dupeRP.setParentProductID(0);
						dupeRP.setParentProduct(dupeProduct);
						dupeProduct.getRelatedProducts().add(dupeRP);
					}
				}
			}
			return dupeProduct;
		}
		return null;
	}

	public Attribute duplicateAttribute(Attribute attribute, Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Duplicating attribute");
		String codePrefix = (String) parameters.get("codePrefix");
		String cloneProducts = (String) parameters.get("cloneProducts");
		AttributeDAO attributeDAO =  getDaoFactory().createDAO(AttributeDAO.class);
		if (attribute != null) {
			Attribute dupeAttribute =  getBusinessObjectFactory().createObject(Attribute.class);
			BeanUtils.copyProperties(dupeAttribute, attribute);
			dupeAttribute.setAttributeID(0);
			dupeAttribute.setAttributeCode(codePrefix + attribute.getAttributeCode());
			dupeAttribute.setProductAttributes(null);

			// Keep track of the default option for below
			Option defaultOption = null;
			if (attribute.getAttributeOptions() != null && !attribute.getAttributeOptions().isEmpty()) {
				dupeAttribute.setAttributeOptions(new LinkedHashSet());
				Iterator aos = attribute.getAttributeOptions().iterator();
				while (aos.hasNext()) {
					Option o = (Option) aos.next();
					Option dupeOption =  getBusinessObjectFactory().createObject(Option.class);
					BeanUtils.copyProperties(dupeOption, o);
					dupeOption.setOptionID(0);
					dupeOption.setAttribute(dupeAttribute);
					dupeAttribute.getAttributeOptions().add(dupeOption);

					// Keep track of the default option for below
					if (attribute.getDefaultOptionID() != null
							&& o.getOptionID() == attribute.getDefaultOptionID().intValue()) {
						defaultOption = dupeOption;
					}
				}
			}
			if (dupeAttribute != null) {

				// Assign existing products to the new attribute
				if (cloneProducts != null && cloneProducts.equals("assign")) {
					if (attribute.getProductAttributes() != null && !attribute.getProductAttributes().isEmpty()) {
						dupeAttribute.setProductAttributes(new LinkedHashSet());
						Iterator pas = attribute.getProductAttributes().iterator();
						while (pas.hasNext()) {
							ProductAttribute pa = (ProductAttribute) pas.next();
							ProductAttribute dupePA = (ProductAttribute) getBusinessObjectFactory().createObject(
									"productAttributeImplementer");
							BeanUtils.copyProperties(dupePA, pa);
							dupePA.setProductAttributeID(0);
							dupePA.setAttributeID(0);
							dupePA.setAttribute(dupeAttribute);
							dupeAttribute.getProductAttributes().add(dupePA);
						}
					}
				}

				// Insert the attribute
				attributeDAO.setAttribute(dupeAttribute);
				attributeDAO.insertAttribute(true);

				// If there was a default option update the attribute with its
				// new ID
				if (defaultOption != null) {
					dupeAttribute.setDefaultOptionID(new Integer(defaultOption.getOptionID()));
					attributeDAO.updateAttribute(true);
				}
			}
			return dupeAttribute;
		}
		return null;
	}

	public Product loadProductFromCode(Product product) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading product: " + product.getCode());
		ProductDAO productDAO =  getDaoFactory().createDAO(ProductDAO.class);
		// Load just the product object, regardless of if it's active
		productDAO.setProduct(product);
		productDAO.loadProductFromCode();
		if (productDAO.getProduct() == null) {
			return product;
		}
		product = productDAO.getProduct();
		product.addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
		return product;
	}
	
}
