package com.softslate.commerce.businessobjects.product;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.events.ChangingSKUEvent;
import com.softslate.commerce.daos.product.AttributeGatewayDAO;
import com.softslate.commerce.daos.product.ProductGatewayDAO;
import com.softslate.commerce.daos.product.SKUDAO;
import com.softslate.commerce.daos.product.SKUGatewayDAO;

/**
 * Processes business logic having to do with skus. Created in the Struts layer
 * in response to requests for viewing, editing, adding, and deleting skus.
 * <p>
 * <code>BasicSKUProcessor</code> is the default implementation of the
 * <code>SKUProcessor</code> interface for the application.
 * 
 * @author David Tobey
 */
public class BasicSKUProcessor extends BaseBusinessProcessor implements
		SKUProcessor {

	static Log log = LogFactory.getLog(BasicSKUProcessor.class);

	public Map loadSKUsAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading skus and count");
		SKUGatewayDAO gatewayDAO = (SKUGatewayDAO) getDaoFactory().createDAO(
				"skuGatewayDAOImplementer");
		return gatewayDAO.loadSKUsAndCount(parameters);
	}

	public Map addSKU(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String skuNumber = (String) parameters.get("skuNumber");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding sku: " + skuNumber);
		}
		Map results = new HashMap();

		// Check that another sku with the same info doesn't exist
		SKU sku = (SKU) getBusinessObjectFactory().createObject(
				"skuImplementer");
		BeanUtils.copyProperties(sku, parameters);
		results = validateSKU(sku);
		if (results != null && results.get("resultCode") != null
				&& !results.get("resultCode").equals("0")) {
			return results;
		}

		// Add the SKU
		if (sku.getProductID() != null && sku.getProductID().intValue() != 0) {
			Product product =  getBusinessObjectFactory()
					.createObject(Product.class);
			product.setProductID(sku.getProductID().intValue());
			sku.setProduct(product);
		}
		SKUDAO dao =  getDaoFactory().createDAO(SKUDAO.class);
		dao.setSKU(sku);
		dao.insertSKU(true);
		results.put("newRecord", dao.getSKU());

		return results;
	}

	public SKU loadSKUFromID(Map parameters) throws Exception {
		return loadSKUFromID((Integer) ConvertUtils.convert(parameters.get("skuID"), Integer.class));
	}

	public SKU loadSKUFromID(int skuID) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading sku from id");
		SKU sku =  getBusinessObjectFactory().createObject(SKU.class);
		SKUDAO dao =  getDaoFactory().createDAO(SKUDAO.class);
		sku.setSkuID(skuID);
		dao.setSKU(sku);
		dao.loadSKUFromID();
		return dao.getSKU();
	}

	public SKU loadSKUFromNumber(Map parameters) throws Exception {
		return loadSKUFromNumber((String) parameters.get("skuNumber"));
	}

	public SKU loadSKUFromNumber(String skuNumber) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading sku from skuNumber");
		SKU sku =  getBusinessObjectFactory().createObject(SKU.class);
		SKUDAO dao =  getDaoFactory().createDAO(SKUDAO.class);
		sku.setSkuNumber(skuNumber);
		dao.setSKU(sku);
		dao.loadSKUFromSKUNumber();
		return dao.getSKU();
	}

	public Map editSKU(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String skuNumber = (String) parameters.get("skuNumber");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing sku: " + skuNumber);
		}
		Map results = new HashMap();

		// Check that another SKU with the same number doesn't exist
		if (parameters.get("skuID") != null) {
			SKU sku =  getBusinessObjectFactory().createObject(SKU.class);
			BeanUtils.copyProperties(sku, parameters);
			results = validateSKU(sku);
			if (results != null && results.get("resultCode") != null && !results.get("resultCode").equals("0")) {
				return results;
			}
		}

		// Load the record
		SKU sku = null;
		if (parameters.get("skuID") != null) {
			sku = loadSKUFromID(parameters);
		}
		if (sku == null && parameters.get("skuNumber") != null) {
			sku = loadSKUFromNumber(parameters);
		}
		results = postChangingSKUEvent(sku, parameters);

		// Update the SKU
		BeanUtils.copyProperties(sku, parameters);
		if (parameters.get("productID") != null && !parameters.get("productID").equals("")
				&& !parameters.get("productID").equals("0")) {
			Product product =  getBusinessObjectFactory().createObject(Product.class);
			product.setProductID(sku.getProductID().intValue());
			sku.setProduct(product);
		} else if (parameters.get("productID") != null
				&& (parameters.get("productID").equals("") || parameters.get("productID").equals("0"))) {
			sku.setProduct(null);
		}
		SKUDAO dao =  getDaoFactory().createDAO(SKUDAO.class);
		dao.setSKU(sku);
		dao.updateSKU(true);
		return results;
	}

	public Map postChangingSKUEvent(SKU sku, Map parameters) throws Exception {
		Map results = new HashMap();
		ChangingSKUEvent event = getBusinessObjectFactory().createObject(ChangingSKUEvent.class);
		event.setBusinessProcessor(this);
		event.setSKU(sku);
		event.setIncomingParameters(parameters);
		event.setResults(results);
		getEventBus().post(event);
		return event.getResults();
	}

	public Map editDeleteSKUs(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple skus");
		SKUGatewayDAO dao =  getDaoFactory().createDAO(SKUGatewayDAO.class);
		Map results = dao.editDeleteSKUs(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deleteSKU(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String skuNumber = (String) parameters.get("skuNumber");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting sku: " + skuNumber);
		}
		SKU sku = (SKU) getBusinessObjectFactory().createObject(
				"skuImplementer");
		BeanUtils.copyProperties(sku, parameters);
		SKUDAO dao =  getDaoFactory().createDAO(SKUDAO.class);
		dao.setSKU(sku);
		dao.deleteSKU(true);
		return;
	}

	public Map validateSKU(SKU sku) throws Exception {
		Map results = new HashMap();

		// Check for a duplicate sku number
		SKUDAO dao =  getDaoFactory().createDAO(SKUDAO.class);
		dao.setSKU(sku);
		dao.loadSKUFromSKUNumber();
		if (dao.getSKU() != null) {
			dao.getDaoFactory().getSession().evict(dao.getSKU());
		}
		boolean insertingButExists = sku.getSkuID() == 0
				&& dao.getSKU() != null && dao.getSKU().getSkuID() > 0;
		boolean updatingButExists = sku.getSkuID() > 0 && dao.getSKU() != null
				&& sku.getSkuID() != dao.getSKU().getSkuID();
		if (insertingButExists || updatingButExists) {
			results.put("resultCode", "DuplicateRecord");
			results.put("duplicateField", "skuNumber");
			results.put("existingRecord", dao.getSKU());
		}

		// Check for a duplicate sku product
		if (sku.getProductID() != null
				&& sku.getProductID().intValue() > 0
				&& (sku.getAttributeID1() == null || sku.getAttributeID1()
						.intValue() == 0)
				&& (sku.getAttributeID2() == null || sku.getAttributeID2()
						.intValue() == 0)
				&& (sku.getAttributeID3() == null || sku.getAttributeID3()
						.intValue() == 0)
				&& (sku.getAttributeID4() == null || sku.getAttributeID4()
						.intValue() == 0)
				&& (sku.getAttributeID5() == null || sku.getAttributeID5()
						.intValue() == 0)
				&& (sku.getOptionID1() == null || sku.getOptionID1().intValue() == 0)
				&& (sku.getOptionID2() == null || sku.getOptionID2().intValue() == 0)
				&& (sku.getOptionID3() == null || sku.getOptionID3().intValue() == 0)
				&& (sku.getOptionID4() == null || sku.getOptionID4().intValue() == 0)
				&& (sku.getOptionID5() == null || sku.getOptionID5().intValue() == 0)) {
			dao =  getDaoFactory().createDAO(SKUDAO.class);
			dao.setSKU(sku);
			dao.loadProductSKU();
			if (dao.getSKU() != null) {
				dao.getDaoFactory().getSession().evict(dao.getSKU());
			}
			insertingButExists = sku.getSkuID() == 0 && dao.getSKU() != null
					&& dao.getSKU().getSkuID() > 0;
			updatingButExists = sku.getSkuID() > 0 && dao.getSKU() != null
					&& sku.getSkuID() != dao.getSKU().getSkuID();
			if (insertingButExists || updatingButExists) {
				results.put("resultCode", "DuplicateRecord");
				results.put("duplicateField", "productID");
				results.put("existingRecord", dao.getSKU());
			}
		}
		return results;
	}

	public Map buildProductSKUs(int productID) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Building/refreshing product skus for product ID: "
					+ productID);
		Map results = new HashMap();
		int insertCount = 0;
		int existsCount = 0;
		Collection skuNumberConflicts = new ArrayList();
		SKUDAO skuDao =  getDaoFactory().createDAO(SKUDAO.class);
		ProductProcessor productProcessor =  getBusinessObjectFactory()
				.createObject(ProductProcessor.class);
		Map parameters = new HashMap();
		parameters.put("productID", Integer.toString(productID));
		Product product = productProcessor.loadProductFromID(parameters);

		// Create the main product SKU
		SKU mainSku = (SKU) getBusinessObjectFactory().createObject(
				"skuImplementer");
		mainSku.setProductID(new Integer(product.getProductID()));
		mainSku.setSkuNumber(product.getCode());
		skuDao.setSKU(mainSku);
		skuDao.loadProductSKU();
		if (skuDao.getSKU() != null) {
			existsCount++;
		} else {
			skuDao.setSKU(mainSku);
			skuDao.loadSKUFromSKUNumber();
			if (skuDao.getSKU() != null) {
				skuNumberConflicts.add(mainSku.getSkuNumber());
			} else {
				mainSku = (SKU) getBusinessObjectFactory().createObject(
						"skuImplementer");
				mainSku.setProduct(product);
				mainSku.setProductID(new Integer(product.getProductID()));
				mainSku.setSkuNumber(product.getCode());
				mainSku.setSkuName(product.getName());
				mainSku.setTrackInventory(false);
				skuDao.setSKU(mainSku);
				skuDao.insertSKU();
				insertCount++;
			}
		}

		// Create SKUs for each attribute/option combination
		Collection matrix = productProcessor.buildMatrix(product);
		int index = 0;
		Iterator i = matrix.iterator();
		while (i.hasNext()) {
			Collection row = (Collection) i.next();
			index++;

			// Create the SKU from a row of the matrix
			SKU sku = (SKU) getBusinessObjectFactory().createObject(
					"skuImplementer");
			sku.setProduct(product);
			sku.setProductID(new Integer(product.getProductID()));
			int count = 1;
			StringBuffer code = new StringBuffer(product.getCode());
			StringBuffer name = new StringBuffer(product.getName());
			Iterator j = row.iterator();
			while (j.hasNext() && count <= 5) {
				Option o = (Option) j.next();
				String attributeProp = "attributeID" + count;
				String optionProp = "optionID" + count;
				PropertyUtils.setProperty(sku, attributeProp, new Integer(o
						.getAttributeID()));
				PropertyUtils.setProperty(sku, optionProp, new Integer(o
						.getOptionID()));
				code.append("|");
				code.append(o.getAttribute().getAttributeCode());
				code.append("=");
				code.append(o.getOptionCode());
				name.append("|");
				name.append(o.getAttribute().getAttributeCode());
				name.append("=");
				name.append(o.getOptionCode());
				count++;
			}

			// Check that the SKU doesn't already exist
			boolean skuExists = false;
			Iterator k = product.getSkus().iterator();
			while (k.hasNext()) {
				SKU s = (SKU) k.next();
				log.debug("Checking against existing sku: "
						+ BeanUtils.describe(s));
				log.debug("Current sku: " + BeanUtils.describe(sku));
				if (s.getProductID().equals(sku.getProductID())
						&& nullOrEqual(s.getAttributeID1(), sku
								.getAttributeID1())
						&& nullOrEqual(s.getAttributeID2(), sku
								.getAttributeID2())
						&& nullOrEqual(s.getAttributeID3(), sku
								.getAttributeID3())
						&& nullOrEqual(s.getAttributeID4(), sku
								.getAttributeID4())
						&& nullOrEqual(s.getAttributeID5(), sku
								.getAttributeID5())
						&& nullOrEqual(s.getOptionID1(), sku.getOptionID1())
						&& nullOrEqual(s.getOptionID2(), sku.getOptionID2())
						&& nullOrEqual(s.getOptionID3(), sku.getOptionID3())
						&& nullOrEqual(s.getOptionID4(), sku.getOptionID4())
						&& nullOrEqual(s.getOptionID5(), sku.getOptionID5())) {
					skuExists = true;
					log.info("SKU exists already:" + BeanUtils.describe(sku));
					break;
				}
			}

			// Insert the SKU
			if (skuExists) {
				existsCount++;
			} else {
				String codeString = code.toString();
				if (codeString.length() > 128) {
					codeString = codeString.substring(0, 127) + index;
				}
				sku.setSkuNumber(codeString);

				// Check for a sku number conflict
				skuDao.setSKU(sku);
				skuDao.loadSKUFromSKUNumber();
				if (skuDao.getSKU() != null) {
					skuNumberConflicts.add(skuDao.getSKU().getSkuNumber());
				} else {

					// Insert the sku
					String nameString = name.toString();
					if (nameString.length() > 250) {
						nameString = nameString.substring(0, 250) + index;
					}
					sku.setSkuName(nameString);
					sku.setTrackInventory(false);
					skuDao.setSKU(sku);
					skuDao.insertSKU();
					insertCount++;
				}
			}
		}
		results.put("insertCount", new Integer(insertCount));
		results.put("existsCount", new Integer(existsCount));
		results.put("skuNumberConflicts", skuNumberConflicts);
		results.put("skuConflictsCount", Integer.toString(skuNumberConflicts
				.size()));
		return results;
	}

	public Map buildAllProductSKUs() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Building/refreshing all product skus.");
		Map results = new HashMap();
		int insertCount = 0;
		int existsCount = 0;
		SKUDAO skuDao =  getDaoFactory().createDAO(SKUDAO.class);
		ProductGatewayDAO productDAO =  getDaoFactory()
				.createDAO(ProductGatewayDAO.class);
		Collection products = productDAO.loadAllProducts();
		Iterator i = products.iterator();
		while (i.hasNext()) {
			Product p = (Product) i.next();
			SKU sku = (SKU) getBusinessObjectFactory().createObject(
					"skuImplementer");
			sku.setProductID(new Integer(p.getProductID()));
			skuDao.setSKU(sku);
			skuDao.loadProductSKU();
			sku = skuDao.getSKU();
			if (sku != null) {
				existsCount++;
			} else {
				sku = (SKU) getBusinessObjectFactory().createObject(
						"skuImplementer");
				sku.setProduct(p);
				sku.setProductID(new Integer(p.getProductID()));
				sku.setSkuNumber(p.getCode());
				sku.setSkuName(p.getName());
				sku.setTrackInventory(false);
				skuDao.setSKU(sku);
				skuDao.insertSKU();
				insertCount++;
			}
		}
		results.put("insertCount", new Integer(insertCount));
		results.put("existsCount", new Integer(existsCount));
		return results;
	}

	public Map buildAllAttributeSKUs() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Building/refreshing all attribute skus.");
		ProductProcessor processor =  getBusinessObjectFactory()
				.createObject(ProductProcessor.class);
		Map results = new HashMap();
		Map existing = processor.loadAttributeSKUs();
		if (log.isDebugEnabled())
			log.debug("Existing attribute skus: " + existing);
		int insertCount = 0;
		int existsCount = 0;
		SKUDAO skuDao =  getDaoFactory().createDAO(SKUDAO.class);
		AttributeGatewayDAO attributeDAO =  getDaoFactory()
				.createDAO(AttributeGatewayDAO.class);
		Collection attributes = attributeDAO.loadAllAttributes();
		Iterator i = attributes.iterator();
		while (i.hasNext()) {
			Attribute a = (Attribute) i.next();
			Collection options = a.getAttributeOptions();
			if (options == null || options.isEmpty()) {
				String key = a.getAttributeID() + "|0|";
				if (existing.get(key) != null) {
					existsCount++;
				} else {
					SKU sku = (SKU) getBusinessObjectFactory().createObject(
							"skuImplementer");
					sku.setAttributeID1(new Integer(a.getAttributeID()));
					sku.setSkuNumber(a.getAttributeCode());
					sku.setSkuName(a.getAttributeName());
					sku.setTrackInventory(false);
					skuDao.setSKU(sku);
					skuDao.insertSKU();
					insertCount++;
				}
			} else {
				Iterator j = options.iterator();
				while (j.hasNext()) {
					Option o = (Option) j.next();
					String key = a.getAttributeID() + "|" + o.getOptionID()
							+ "|";
					if (existing.get(key) != null) {
						existsCount++;
					} else {
						SKU sku =  getBusinessObjectFactory()
								.createObject(SKU.class);
						sku.setAttributeID1(new Integer(a.getAttributeID()));
						sku.setOptionID1(new Integer(o.getOptionID()));
						sku.setSkuNumber(a.getAttributeCode() + "="
								+ o.getOptionCode());
						sku.setSkuName(a.getAttributeName() + "="
								+ o.getOptionName());
						sku.setTrackInventory(false);
						skuDao.setSKU(sku);
						skuDao.insertSKU();
						insertCount++;
					}

				}
			}
		}
		results.put("insertCount", new Integer(insertCount));
		results.put("existsCount", new Integer(existsCount));
		return results;
	}

	public boolean nullOrEqual(Object o1, Object o2) {
		return ((o1 == null && o2 == null) || (o1 != null && o1.equals(o2)) || (o2 != null && o2
				.equals(o1)));
	}

	public int loadMaxSKUID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading max sku ID.");
		}
		SKUGatewayDAO gatewayDAO = (SKUGatewayDAO) getDaoFactory().createDAO(
				"skuGatewayDAOImplementer");
		return gatewayDAO.loadMaxSKUID();
	}
}
