package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface for business logic methods affecting products and other related
 * objects.
 * <p>
 * Instances implementing <code>ProductProcessor</code> are usually created in
 * the Struts layer in response to a request from a user for product
 * information. The <code>ProductProcessor</code> object uses data access
 * objects to look up the information, and return it to the Struts layer.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ProductProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "productProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "productProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.BasicProductProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface ProductProcessor extends BusinessObject {

	/**
	 * Retrieves product information from the database. In the default
	 * implementation, this method calls
	 * {@link com.softslate.commerce.daos.product.ProductDAO#loadProduct()} to
	 * populate the incoming <code>Product</code> using its <code>code</code>.
	 * 
	 * @param product
	 *            A <code>Product</code> object whose <code>code</code>
	 *            property has been populated.
	 * @return A <code>Product</code> object with all its properties populated
	 *         from the database. If the product does not exist in the database,
	 *         <code>null</code>.
	 * @throws Exception
	 */
	abstract Product getProductFromCode(Product product) throws Exception;

	/**
	 * Retrieves product information from the database. In the default
	 * implementation, this method calls
	 * {@link com.softslate.commerce.daos.product.ProductDAO#loadProductFromSeoCode()}
	 * to populate the incoming <code>Product</code> using its
	 * <code>seoCode</code>.
	 * 
	 * @param product
	 *            A <code>Product</code> object whose <code>seoCode</code>
	 *            property has been populated.
	 * @return A <code>Product</code> object with all its properties populated
	 *         from the database. If the product does not exist in the database,
	 *         <code>null</code>.
	 * @throws Exception
	 */
	abstract Product getProductFromSeoCode(Product product) throws Exception;

	/**
	 * Retrieves all SKUs associated with attributes but not products.
	 * 
	 * @return A <code>Map</code> of all the <code>SKU</code>s in the
	 *         system associated with attributes but not products.
	 * @throws Exception
	 */
	abstract Map loadAttributeSKUs() throws Exception;

	/**
	 * Retrieves all SKUs associated with attributes but not products, as a
	 * Collection of SKUs.
	 * 
	 * @return A <code>Collection</code> of all the <code>SKU</code>s in
	 *         the system associated with attributes but not products.
	 * @throws Exception
	 */
	abstract Collection loadAttributeSKUCollection() throws Exception;

	/**
	 * Retrieves category information from the database. In the default
	 * implementation, this method calls getCategoryFromCode(Category category,
	 * boolean loadProducts) with the loadProducts parameter set to true.
	 * category tree.
	 * 
	 * @param category
	 *            A <code>Category</code> object whose <code>code</code>
	 *            property has been populated.
	 * @return A <code>Category</code> object with all its properties
	 *         populated from the database. If the category does not exist in
	 *         the database, <code>null</code>.
	 * @throws Exception
	 */
	abstract Category getCategoryFromCode(Category category) throws Exception;

	/**
	 * Retrieves category information from the database. In the default
	 * implementation, this method calls
	 * {@link com.softslate.commerce.daos.product.CategoryDAO#loadCategory()} to
	 * populate the incoming <code>Category</code> using its <code>code</code>.
	 * <p>
	 * If the loadProducts parameter is set to true, in addition to all of the
	 * category's regular properties, its <code>productListCollection</code>
	 * is populated with just those <code>Product</code>s falling on the
	 * current page being displayed. The paging parameters for the number of
	 * items to display and which page to display come from the incoming
	 * <code>Category</code>'s <code>itemsPerPage</code> and
	 * <code>firstRow</code> properties, respectively.
	 * <p>
	 * The method
	 * {@link com.softslate.commerce.daos.product.ProductGatewayDAO#loadProductList(ProductList)}
	 * is used to pull in the category's products.
	 * <p>
	 * Also in the default implementation, the <code>Category</code>'s
	 * <code>parentCollection</code> and <code>parentString</code>
	 * properties are set by calling
	 * {@link com.softslate.commerce.daos.product.CategoryDAO#loadParents()}.
	 * These properties are used in breadcrumbing and in the display of the
	 * category tree.
	 * 
	 * @param category
	 *            A <code>Category</code> object whose <code>code</code>
	 *            property has been populated.
	 * @param loadProducts
	 *            If set to true, the category's products will be loaded into
	 *            the category's <code>productListCollection</code> property.
	 * @return A <code>Category</code> object with all its properties
	 *         populated from the database. If the category does not exist in
	 *         the database, <code>null</code>.
	 * @throws Exception
	 */
	abstract Category getCategoryFromCode(Category category,
			boolean loadProducts) throws Exception;

	/**
	 * Retrieves a <code>Collection</code> representing the store's category
	 * tree. Each item in the <code>Collection</code> is an instance of
	 * {@link com.softslate.commerce.businessobjects.product.Category}, which
	 * corresponds to each one of the "top level" categories defined in the
	 * store (i.e., categories whose <code>parentCategory</code> property is
	 * <code>null</code>). To render the tree, iterate through the top level
	 * categories and recursively navigate through each of their
	 * <code>subcategoryCollection</code>s in turn.
	 * <p>
	 * The default implementation of this method calls
	 * {@link com.softslate.commerce.daos.product.ProductGatewayDAOHibernate#loadCategoryTree()},
	 * which uses {@link org.hibernate.Hibernate#initialize(java.lang.Object)}
	 * to pull the entire tree from the database non-lazily.
	 * 
	 * @return A <code>Collection</code> representing the store's category
	 *         tree.
	 * @throws Exception
	 */
	abstract Collection loadCategoryTree() throws Exception;

	/**
	 * Retrieves a <code>Collection</code> containing built-in categories for
	 * the store. Each item in the <code>Collection</code> is an instance of
	 * {@link com.softslate.commerce.businessobjects.product.Category}.
	 * Built-in categories are typically composed of specials or other featured
	 * products that need to displayed on arbitrary pages thoughout the store.
	 * <p>
	 * Built-in categories are definined by the "builtInCategories" setting in
	 * the <code>sscSetting</code> database table, a comma-separated list of
	 * category codes.
	 * 
	 * @return A <code>Collection</code> of <code>Category</code> objects
	 *         representing the store's built-in categories.
	 * @throws Exception
	 */
	abstract Collection loadBuiltInCategories() throws Exception;

	/**
	 * Retrieves a <code>Collection</code> containing categories corresponding
	 * to the codes of the argument. Each item in the <code>Collection</code>
	 * is an instance of
	 * {@link com.softslate.commerce.businessobjects.product.Category}. This
	 * method is used to populate the 'headerCategoriesAndContentPages' and
	 * 'footerCategoriesAndContentPages' - the categories that appear as links
	 * in the header and footer of the store. 
	 * 
	 * @return A <code>Collection</code> of <code>Category</code> objects
	 *         representing the store's built-in categories.
	 * @throws Exception
	 */
	abstract Collection loadCategories(String categoryList) throws Exception;

	/**
	 * Retrieves information for a list of products from the database. Depending
	 * on the value of the <code>productListType</code> for the
	 * <code>ProductList</code>, the products could be the entire list of
	 * products in the system, the products under a given category, or the
	 * products matching a given search request.
	 * 
	 * @param productList
	 *            A <code>ProductList</code> object whose
	 *            <code>itemsPerPage</code>,<code>firstCode</code>,
	 *            <code>productListType</code>, and optionally
	 *            <code>categoryID</code> or <code>searchString</code> and
	 *            <code>stopWords</code> properties have been set.
	 * @return A <code>ProductList</code> object with all its properties
	 *         populated from the database.
	 * @throws Exception
	 */
	abstract ProductList getProductList(ProductList productList)
			throws Exception;

	abstract Collection loadAllProducts() throws Exception;

	abstract int loadMaxProductID() throws Exception;

	abstract Collection loadProductsForIndexing(int firstProductID,
			int lastProductID) throws Exception;

	abstract Collection loadAllProductNamesCodesIDs() throws Exception;

	abstract Collection loadActiveProductNamesCodesIDs() throws Exception;

	abstract Map loadProductsAndCount(Map parameters) throws Exception;

	abstract Map addProduct(Map parameters) throws Exception;

	abstract Product loadProductFromID(Map parameters) throws Exception;

	abstract Map editProduct(Map parameters) throws Exception;

	abstract Map editDeleteProducts(Map parameters) throws Exception;

	abstract Map deleteProduct(Map parameters) throws Exception;

	abstract HashMap generateSettingsMap(Collection settings) throws Exception;

	abstract Collection getAllAttributesAndOptions(int productID)
			throws Exception;

	abstract Collection loadSettings(Map properties) throws Exception;

	abstract Collection processSettings(Map properties) throws Exception;

	abstract Collection buildMatrix(Product product);

	abstract Map sortProductsByName(Map parameters) throws Exception;

	abstract Map sortProductsByCode(Map parameters) throws Exception;

	abstract Map cloneProduct(Map parameters) throws Exception;

	abstract Map createCategoryMapFromCategoryTree(Collection categoryTree);

	abstract Product loadProductFromCode(Product product) throws Exception;

	abstract Map postChangingProductEvent(Product product, Map parameters) throws Exception;

	abstract Product loadProductFromCode(Map parameters) throws Exception;

	abstract Product loadProductFromCode(String code) throws Exception;
}
