package com.softslate.commerce.daos.core;

import java.sql.Connection;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import javax.inject.Inject;
import javax.inject.Named;
import javax.sql.DataSource;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Filter;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.jdbc.Work;

import com.google.inject.Injector;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactoryImpl;
import com.softslate.commerce.businessobjects.core.Settings;

/**
 * Abstract superclass for all objects in the data access layer.
 * <p>
 * The methods provided allow implementations to communicate with other elements
 * of the application. In particular, retrieve settings (using
 * {@link #getSettings()}), retrieve database connections (using
 * {@link #getConnection()}), retrieve SQL statements (using
 * {@link #getSqlMap()} and {@link #getSQL}), and create other data
 * access objects (using {@link #getDaoFactory()}).
 * 
 * @author David Tobey
 * 
 */
public abstract class BaseDAO implements DAOInterface {

    static Log log = LogFactory.getLog(BaseDAO.class);

    private Settings settings = null;

    public Settings getSettings() {
        return (this.settings);
    }

	@Inject
    public void setSettings(Settings settings) {
        this.settings = settings;
    }

    private Properties appSettings = null;

    public Properties getAppSettings() {
        return appSettings;
    }

	@Inject
	public void setAppSettings(@Named("appSettings") Properties appSettings) {
        this.appSettings = appSettings;
    }

    private Properties appComponents = null;

    public Properties getAppComponents() {
        return appComponents;
    }

	@Inject
	public void setAppComponents(@Named("appComponents") Properties appComponents) {
        this.appComponents = appComponents;
    }

    private DataSource dataSource = null;

    public DataSource getDataSource() {
        return (this.dataSource);
    }

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    private Connection connection = null;

    public Connection getConnection() {
        return (this.connection);
    }

    public void setConnection(Connection connection) {
        this.connection = connection;
    }

    private Map sqlMap = new HashMap();

    public Map getSqlMap() {
        return (this.sqlMap);
    }

    public void setSqlMap(Map sqlMap) {
        this.sqlMap = sqlMap;
    }

    private Map importExportSqlMap = new HashMap();

    public Map getImportExportSqlMap() {
        return importExportSqlMap;
    }

    public void setImportExportSqlMap(Map importExportSqlMap) {
        this.importExportSqlMap = importExportSqlMap;
    }

    private DAOFactory daoFactory = null;

    public DAOFactory getDaoFactory() {
        return (this.daoFactory);
    }

	@Inject
    public void setDaoFactory(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
    }

	private Injector injector = null;

	public Injector getInjector() {
		return injector;
	}

	@Inject
	public void setInjector(Injector injector) {
		this.injector = injector;
	}

    public void initialize(Map transferMap) throws Exception {
        PropertyUtils.copyProperties(this, transferMap);
        return;
    }

    /**
     * 
     * Legacy method that returns an SQL statement configured in the .properties
     * files under "/WEB-INF/classes/resources".
     * 
     * @param module
     *            The application module corresponding to the desired SQL
     *            statement, such as "core", "product", "order", or "customer".
     * @param key
     *            A string key for the SQL statement corresponding to the keys
     *            in the .properties files.
     * @return An SQL statement, or <code>null</code> if the requested
     *         statement could not be found.
     * @throws Exception
     */
    public String getSQL(String module, String key) throws Exception {
        Properties moduleProperties = (Properties) getSqlMap().get(module);
        return moduleProperties.getProperty(key);
    }

    /**
     * 
     * Returns an SQL statement configured in the .properties files under
     * "/WEB-INF/classes/resources/importexport".
     * 
     * @param module
     *            The application module corresponding to the desired SQL
     *            statement. Must be "importexport".
     * @param key
     *            A string key for the SQL statement corresponding to the keys
     *            in the .properties files.
     * @return An SQL statement, or <code>null</code> if the requested
     *         statement could not be found.
     * @throws Exception
     */
    public String getImportExportSQL(String module, String key)
            throws Exception {
        Properties moduleProperties = (Properties) getImportExportSqlMap().get(
                module);
        return moduleProperties.getProperty(key);
    }

    /**
     * Formats a Date object into a string suitable for storage in the database.
     * 
     * @param dateTime
     * @return a String in the form of "yyyy-MM-dd HH:mm:ss:SSS" suitable for
     *         storage in the database.
     */
    public String formatDateTime(Date dateTime) {
        SimpleDateFormat formatter = new SimpleDateFormat(
                "yyyy-MM-dd HH:mm:ss:SSS", Locale.getDefault());
        return formatter.format(dateTime);
    }

    /**
     * Formats a Date object into a string suitable for storage in the database.
     * 
     * @param dateTime
     * @return a String in the form of "yyyy-MM-dd HH:mm:ss:SSS" suitable for
     *         storage in the database.
     */
    public Date parseDateTime(String dateTime) throws Exception {
        SimpleDateFormat formatter = new SimpleDateFormat(
                "yyyy-MM-dd HH:mm:ss:SSS", Locale.getDefault());
        return formatter.parse(dateTime);
    }

    private DAOUtils daoUtils = null;

    public DAOUtils getDAOUtils() {
        if (daoUtils != null) {
            return daoUtils;
        }
        BusinessObjectFactoryImpl bof = new BusinessObjectFactoryImpl();
        bof.setAppComponents(getAppComponents());
        daoUtils = (DAOUtils) bof.createUtilsObject("daoUtilsImplementer");
        return daoUtils;
    }

    public void setDAOUtils(DAOUtils daoUtils) {
        this.daoUtils = daoUtils;
    }

    /**
     * Returns a custom named query with the "-custom" suffix if one exists,
     * otherwise returns the normally named query.
     * <p>
     * Named queries can be overridden by creating a new query in one of the
     * <code>queries-custom.hbm.xml</code> files. The new query must have the
     * same name as the query it's replacing, but with "-custom" appended to the
     * name. This method looks to see if a custom query exists before returning
     * the regular query.
     * 
     * @param session
     *            The current Hibernate session.
     * @param queryName
     *            The name of the query to return (without the "-custom"
     *            suffix).
     * @return A Query object.
     * @throws HibernateException
     */
    public Query getNamedQuery(Session session, String queryName)
            throws HibernateException {
        Query q = null;
        try {
            q = session.getNamedQuery(queryName + "-custom");
            if (log.isInfoEnabled())
                log.info("Found custom named query: " + queryName + "-custom");
        } catch (HibernateException he) {
            q = session.getNamedQuery(queryName);
        }
        return q;
    }

	public void evictCache() {
		getDaoFactory().evictCache();
	}
	
	public Object loadObject(Object object) throws Exception
	{
		return object;
	}
	
	public Object updateObject(Object object, boolean commit) throws Exception
	{
		return object;
	}
	
	public boolean deleteObject(Object object, boolean commit) throws Exception
	{
		return false;
	}
	
	public Object insertObject(Object object, boolean commit) throws Exception
	{
		return object;
	}
	
	public Collection loadAll() throws Exception
	{
		return null;
	}
	
	public Collection loadAll(Filter filter) throws Exception
	{
		return null;
	}

	public void doWorkToSetConnection(Session session) {
		session.doWork(new Work() {
			public void execute(Connection connection) throws SQLException {
				setConnection(connection);
			}
		});
	}
}
